import unittest
from openProduction.suite import TestSuite, TestClass, TestSuiteValueContainer
from openProduction.suite import ExecutionHelpers
from openProduction.common import Types, Version

class MyValues(TestSuiteValueContainer.TestSuiteValues):
    def __init__(self):
        super(MyValues, self).__init__()
        self.addValue(Types.String("MY_VAL", "Simple description"))

class MyCls(TestClass.TestClass):
    def __init__(self):
        self.params = {}
        self.params["startTrigger"] = False
        
    def _myFunc(self, params, values):
        self.assertEqual(0,0)
    
    @ExecutionHelpers.skipIf(True==True)
    def _mySkipFunc(self, params, values):
        pass
    
    def _myFailFunc(self, params, values):
        self.assertEqual(0,1)
        
    def _myExceptionFunc(self, params, values):
        raise RuntimeError("something went wrong...")
        
    def _myFuncWithParams(self, params, values):
        self.assertEqual(params["TEST_PARAM"], 10)
        
    def _myCustomVals(self, params, values):
        values.MY_VAL = "hello"
        
    def _myAddTestGroup(self, params, attrs):
        tg = TestSuite.TestCaseGroup("testGroup", "testDoc")
        _myFunc = MyCls()._myFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg.addTestCase(tc)
        return tg

class TestSuiteTests(unittest.TestCase):

    def test_pass(self):
        """Test a pass"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        ts = TestSuite.TestSuite(name, version, "DOC")
        
        _myFunc = MyCls()._myFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.numTotal, 1)
        self.assertEqual(res.numGood, 1)
        self.assertEqual(res.numFails, 0)
        self.assertEqual(res.numErrors, 0)
        self.assertEqual(res.numSkips, 0)
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 1)
    
    def test_param(self):
        """Test a pass"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        myParams = {}
        myParams["TEST_PARAM"] = 10
        ts = TestSuite.TestSuite(name, version, "DOC", params=myParams)
        
        _myFunc = MyCls()._myFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.numTotal, 1)
        self.assertEqual(res.numGood, 1)
        self.assertEqual(res.numFails, 0)
        self.assertEqual(res.numErrors, 0)
        self.assertEqual(res.numSkips, 0)
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 1)
    
    def test_skip(self):
        """Test a skip"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        ts = TestSuite.TestSuite(name, version, "DOC")
        
        _myFunc = MyCls()._mySkipFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.numTotal, 1)
        self.assertEqual(res.numGood, 0)
        self.assertEqual(res.numFails, 0)
        self.assertEqual(res.numErrors, 0)
        self.assertEqual(res.numGood, 0)
        self.assertEqual(res.numSkips, 1)

    def test_fail(self):
        """Test a fail"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        ts = TestSuite.TestSuite(name, version, "DOC")
        
        _myFunc = MyCls()._myFailFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.numTotal, 1)
        self.assertEqual(res.numGood, 0)
        self.assertEqual(res.numFails, 1)
        self.assertEqual(res.numErrors, 0)
        self.assertEqual(res.numGood, 0)
        self.assertEqual(res.numSkips, 0)
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 0)
        
    def test_error(self):
        """Test an exception"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        ts = TestSuite.TestSuite(name, version, "DOC")
        
        _myFunc = MyCls()._myExceptionFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.numTotal, 1)
        self.assertEqual(res.numGood, 0)
        self.assertEqual(res.numFails, 0)
        self.assertEqual(res.numErrors, 1)
        self.assertEqual(res.numGood, 0)
        self.assertEqual(res.numSkips, 0)
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 0)
        
    def test_values(self):
        """test values"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        ts = TestSuite.TestSuite(name, version, "DOC")
        
        _myFunc = MyCls()._myFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.suite.values.getValue("VERSION"), version.toFullString())
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 1)
        
    def test_custom_values(self):
        """test custom values"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        atts = MyValues()
        
        ts = TestSuite.TestSuite(name, version, "DOC", params=None, values=atts)
        
        _myFunc = MyCls()._myCustomVals
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.suite.values.getValue("VERSION"), version.toFullString())
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 1)
        self.assertEqual(res.suite.values.getValue("MY_VAL"), "hello")
        
        #run a different test, values have to be reset        
        ts = TestSuite.TestSuite(name, version, "DOC", params=None, values=atts)
        _myFunc = MyCls()._myFunc
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.REGULAR)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.suite.values.getValue("VERSION"), version.toFullString())
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 1)
        self.assertEqual(res.suite.values.getValue("MY_VAL"), None)
        
    def test_dynamic_tg_add(self):
        """Test dynamic addition of test cases"""
        name = "myTS"
        version = Version.Version("VERSION1", 123, "http://nothing")
        ts = TestSuite.TestSuite(name, version, "DOC")
        
        _myFunc = MyCls()._myAddTestGroup
        tc = TestSuite.TestCase(_myFunc.__name__, _myFunc.__doc__, _myFunc, TestSuite.StepType.SETUP)
        tg = TestSuite.TestCaseGroup("Group", "GroupDoc")
        tg.addTestCase(tc)
        ts.addTestCaseGroup(tg)
        tr = TestSuite.TestSuiteRunner(ts)
        fut = tr.loadSuite()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        fut = tr.run()
        res = fut.result(timeout=1)
        self.assertEqual(fut.exception(), None)
        
        self.assertEqual(res.numTotal, 2)
        self.assertEqual(res.numGood, 2)
        self.assertEqual(res.numFails, 0)
        self.assertEqual(res.numErrors, 0)
        self.assertEqual(res.numSkips, 0)
        self.assertEqual(res.suite.values.getValue("SUCCESS"), 1)
        self.assertNotEqual(res.stepResults[0].case.func, res.stepResults[1].case.func)
        
if __name__ == '__main__':
    unittest.main()


from enum import Enum
from openProduction.common import Signals, Exceptions
import os
import winsound
import threading

class ImageFormat(Enum):
    GRAYSCALE_8 = 0
    RGB_888 = 1

class IOAnswer(Enum):
    YES = 0
    NO = 1
    CANCEL = 2
    TIMEOUT = 3

class BaseIOHandler:
    def __init__(self):
        self.signalMessage = Signals.Signal()
        self._abort_event = threading.Event()
        
    def _queryYesNo(self, title, msg, timeout=10):
        return IOAnswer.CANCEL
    
    def newImageData(self, image, imageType):
        pass
    
    def message(self, msg, timeout=10):
        self.signalMessage.emit(msg)    
    
    #############################################
    # Methods below should not be overwritten ! #
    #############################################
        
    def abort(self):
        self._abort_event.set()
        
    def abortClear(self):
        self._abort_event.clear()        

    def queryYesNo(self, title, msg, timeout=10):
        if self._abort_event.is_set():
            raise Exceptions.OpenProductionAbortError("")
        else:
            return self._queryYesNo(title, msg, timeout=10)
        
    def playNotify(self):
        if self._abort_event.is_set():
            raise Exceptions.OpenProductionAbortError("")
        myPath = os.path.split(os.path.abspath(__file__))[0]
        soundPath = os.path.abspath(os.path.join(myPath, "../UI/qml/sound/notify.wav"))
        try:
            winsound.PlaySound(soundPath, winsound.SND_FILENAME)
        except:
            pass
        
    def playError(self):
        if self._abort_event.is_set():
            raise Exceptions.OpenProductionAbortError("")
        myPath = os.path.split(os.path.abspath(__file__))[0]
        soundPath = os.path.abspath(os.path.join(myPath, "../UI/qml/sound/error.wav"))
        try:
            winsound.PlaySound(soundPath, winsound.SND_FILENAME)
        except:
            pass
        
    def playSuccess(self):
        if self._abort_event.is_set():
            raise Exceptions.OpenProductionAbortError("")
        myPath = os.path.split(os.path.abspath(__file__))[0]
        soundPath = os.path.abspath(os.path.join(myPath, "../UI/qml/sound/success.wav"))
        try:
            winsound.PlaySound(soundPath, winsound.SND_FILENAME)
        except:
            pass               

class IOMessage:
    def __init__(self, msg):
        self.msg = msg



from concurrent.futures import ThreadPoolExecutor

class Signal:
    
    def __init__(self, asynchron=False):
        self._connectors = []
        self._pool = ThreadPoolExecutor(5)
        self._async = asynchron
    
    def connect(self, cb):
        self._connectors.append(cb)
        
    def disconnect(self, cb=None):
        if cb == None:
            self._connectors = []
        else:
            try:
                self._connectors.remove(cb)
            except:
                pass
        
    def emit(self, *args,**kwargs):
        for c in self._connectors:
            if hasattr(c, "emit"):
                if self._async:
                    self._pool.submit(c.emit, *args, **kwargs)
                else:
                    c.emit(*args, **kwargs)
            else:
                if self._async:
                    self._pool.submit(c, *args, **kwargs)
                else:
                    c(*args, **kwargs)
            
            
if __name__ == "__main__":
    
    def _onConnect(*args, **kwagrs):
        print ("test connect " + args[0])
    def _onConnect2(*args, **kwagrs):
        print ("test connect2 " + args[0])
    def _onConnectSignal2(*args, **kwagrs):
        print ("test chained signal " + args[0])
        
    s2 = Signal()
    s2.connect(_onConnectSignal2)
    s = Signal()
    s.connect(_onConnect)
    s.connect(_onConnect2)
    s.connect(s2)
    s.emit("arg11")


from yapsy.IPlugin import IPlugin
from openProduction.connectors import Config
import traceback
from openProduction.common import misc
import logging
from enum import Enum

PROD_RUN_PRE = "Product_"
PROD_RUN_STAT_PRE = "Station_"

class ConnectorErrors(Enum):
    NO_ERROR = 0
    INVALID_CONFIG = 1
    EXISTS = 2
    FORBIDDEN = 3
    NOT_FOUND = 4
    UNSPECIFIC = 5
    INVALID_PARAM = 6

class BaseConnector(IPlugin):
    def __init__(self, configDescription):
        super(BaseConnector, self).__init__()
        self.configDescription = configDescription
        self.logger = logging.getLogger(misc.getAppName())
        try:
            self.config = Config.Config(configDescription)            
        except Exception as e:
            self.logger.error("loading connector config failed with error message %s"%str(e))
            self.logger.info("full traceback:\n%s"%traceback.format_exc())
            self.config = []
            
    def startTransaction(self, dbName):
        raise NotImplementedError("startTransaction method has to be defined in connector's plugin")
    
    def commitTransaction(self):
        raise NotImplementedError("commitTransaction method has to be defined in connector's plugin")
        
    def rollbackTransaction(self):
        raise NotImplementedError("rollbackTransaction method has to be defined in connector's plugin")
            
    def getValueDataType(self, value):
        raise NotImplementedError("getValueDatatype method has to be defined in connector's plugin")
    
    def createColumn(self, dbName, tableName, columnName, dataTypeString, constraints=None):
        raise NotImplementedError("createDatabase method has to be defined in connector's plugin")
    
    def createDatabase(self, dbName):
        raise NotImplementedError("createDatabase method has to be defined in connector's plugin")
    
    def deleteDatabase(self, dbName):
        raise NotImplementedError("deleteDatabase method has to be defined in connector's plugin")
    
    def createTables(self, dbName, tables):
        raise NotImplementedError("createTables method has to be defined in connector's plugin")
    
    def createTable(self, dbName, tableName, columns):
        raise NotImplementedError("createTable method has to be defined in connector's plugin")
        
    def listDatabases(self):
        raise NotImplementedError("createTable method has to be defined in connector's plugin")

    def addRow(self, dbName, table, entries):
        raise NotImplementedError("addRow method has to be defined in connector's plugin")
        
    def updateRow(self, dbName, table, entries, filt):
        raise NotImplementedError("updateRow method has to be defined in connector's plugin")
        
    def deleteRows(self, dbname, table, filt):
        raise NotImplementedError("deleteRows method has to be defined in connector's plugin")
        
    def getData(self, dbName, table, columns, filt=None):
        raise NotImplementedError("getData method has to be defined in connector's plugin")
    
    def getColumnNames(self, dbName, table):
        raise NotImplementedError("getColumnNames method has to be defined in connector's plugin")
    
    def getColumnContent(self, dbName, table, columnName):
        raise NotImplementedError("getColumnContent method has to be defined in connector's plugin")
        
    def getColumnDataType(self, dbName, table, columnName):
        raise NotImplementedError("getColumnDataType method has to be defined in connector's plugin")
    
    def getDescription(self):
        raise NotImplementedError("getDescription method has to be defined in connector's plugin")
        
    def _onNewConfig(self):
        raise NotImplementedError("_onNewConfig method has to be defined in connector's plugin")
        
    def changeColumnDataType(self, dbName, table, columnName, dataTypeString):
        raise NotImplementedError("changeColumnDataType method has to be defined in connector's plugin")
        
    def saveValues(self, dbName, table, values):
        raise NotImplementedError("saveValues method has to be defined in connector's plugin")
        
    def syncColumns(self, dbName, tableName, values):
        ok = True
        
        if self.dataBaseExists(dbName) == False:
            self.logger.info("database %s doesn't exist. Creating it..."%dbName)            
            ok = self.createDatabase(dbName)            

        if ok and self.tableExists(dbName, tableName) == False:
            self.logger.info("table %s.%s doesn't exist. Creating it..."%(dbName, tableName))            
            ok = self.createTable(dbName, tableName)
        
        anyError = False
        rv, existingColumns = self.getColumnNames(dbName, tableName)
        if rv:
            existingColumns = [x.upper() for x in existingColumns]
            for val in values:
                columnName = val.name
                dataTypeString = self.getValueDataType(val)
                if val.name.upper() not in existingColumns:
                    
                    
                    ok = self.createColumn(dbName, tableName, columnName, dataTypeString, errorIfExists=True)
                    if not ok:
                        anyError = True
                        self.logger.warning("adding column %s to database %s.%s failed"%(columnName, dbName, tableName))
                    else:
                        self.logger.info("added column %s to database %s.%s"%(columnName, dbName, tableName))
                else:
                    rv, dtype = self.getColumnDataType(dbName, tableName, columnName)
                    if rv:
                        if dataTypeString.upper() != dtype:
                            rv = self.changeColumnDataType(dbName, tableName, columnName, dataTypeString)
                            if rv:
                                self.logger.info("changed column %s to datatype %s"%(columnName, dataTypeString))
                            else:
                                self.logger.warning("changing datatype of column %s to datatype %s failed"%(columnName, dataTypeString))
        else:
            anyError = True
            
        if anyError:
            ok = False
        else:
            ok = True
        return ok

    def getDBName(self, product, station):
        return PROD_RUN_PRE + product
    
    def getTableName(self, product, station):
        return PROD_RUN_STAT_PRE + station

    def createProduct(self, product, station):
        err = False
        dbName = self.getDBName(product, station)
        tableName = self.getTableName(product, station)
        
        ok = self.createDatabase(dbName, errorIfExists=False)
        if ok == False:
            err = True
        ok = self.createTable(dbName, tableName, errorIfExists=False)
        
        if err:
            ok = False
            
        return ok
            
        
    def deleteProduct(self, product, station):
        dbName = self.getDBName(product, station)
        return self.deleteDatabase(dbName)
                
    def applyConfig(self, config, workDir):
        rv = ConnectorErrors.NO_ERROR
        self.workDir = workDir
        if self.config != []:
            ok = self.config.fromDict(config)
            if ok:
                rv = self._onNewConfig()
            else:
                rv = ConnectorErrors.INVALID_CONFIG
        return rv
            
    def testConfig(self, config):
        ok = True
        description = "Success"
        
        try:
            c = Config.Config(self.configDescription)
        except Exception as e:
            description = "loading connector config failed with error message %s"%str(e)
            self.logger.error(description)
            self.logger.info("full traceback:\n%s"%traceback.format_exc())
            ok = False
            
        if ok:
            if c.fromJson(config) == False:
                ok = False
                description = c.errmsg()
        
        return ok, description
    
    def getConfig(self):
        return self.config


from openProduction.common import Exceptions
import threading

class TestClass:
    def __init__(self):
        self._abort_event = threading.Event()
        self._unload_event = threading.Event()
        
    def abort(self):
        self._abort_event.set()
        
    def abortClear(self):
        self._abort_event.clear()
        
    def unload(self):
        self._unload_event.set()
        
    def unloadClear(self):
        self._unload_event.clear()
    
    def installRunner(self, runner):
        return True
    
    def install(self, suite):
        return True
    
    def _preAssert(self):
        if self._abort_event.is_set():
            raise Exceptions.OpenProductionAbortError("")
            
    def evalAssert(self, okay, msg):
        self._preAssert()
        if not okay:
            raise Exceptions.OpenProductionAssertionError(msg)
    
    def assertEqual(self, val, comp, msg=None):
        if msg == None:
            msg="Asserion error %s!=%s"%(str(val), str(comp))
        self.evalAssert(val == comp, msg)

    def assertInRange(self, val, minRange, maxRange, msg=None):
        if msg == None:
            msg="Asserion error %s!=[%s...%s]"%(str(val), str(minRange), str(maxRange))
        self.evalAssert(val >= minRange and val <= maxRange, msg)
        
    def assertNotEqual(self, val, comp, msg=None):
        if msg == None:
            msg="Asserion error %s==%s"%(str(val), str(comp))
        self.evalAssert(val != comp, msg)


import logging
import os
from functools import partial
from git import Repo
import traceback
import shutil

from openProduction.station import Station
from openProduction.connectors import ConnectionManager
from openProduction.common import Types, misc

def _checkGitRemote(wdir, stationName, url):
    ok = True
    initNewRepo = False
    logger = logging.getLogger(misc.getAppName())
    if os.path.exists(wdir):
        delFolder = False
        repo = Repo.init(wdir)
        try:
            remote = repo.remote()
            if remote.url != url:
                logger.error("directory %s is a git repo, but remote URL is set to %s"%(wdir, url))
                ok = False
        except:
            logger.info("directory %s exists, but appears to be no valid git repository, \
erasing folder, as user previously agreed to lose changes"%wdir)
            logger.info("full traceback:\n%s"%traceback.format_exc())
            delFolder = True
            
        if delFolder:
            try:
                shutil.rmtree(wdir)
            except:
                logger.error("error deleting folder %s"%wdir)
                logger.info("full traceback:\n%s"%traceback.format_exc())
                ok = False
                
            if ok:
                initNewRepo = True
    else:
        initNewRepo = True
        
    if initNewRepo:
        try:
            repo = Repo.clone_from(url, wdir)
        except:
            logger.error("error cloning git repository from %s"%url)
            logger.info("full traceback:\n%s"%traceback.format_exc())
            ok = False
            
    return ok

class StationInstaller:
    def __init__(self):
        self.logger = logging.getLogger(misc.getAppName())
    
    def _printBanner(self):
        
        licStr = "This project is licensed under %s, see the accompanied VERSION file for more details"%misc.getLicense(misc.License.Type)
        
        print("="*len(licStr))
        print("Welcome to openProduction %s"%misc.getVersion())
        print("")
        print(" * Documentation: %s"%misc.getLink(misc.Links.Documentation))
        print(" * Sources:       %s"%misc.getLink(misc.Links.SourceCode))
        print("")
        print(licStr)
        print("="*len(licStr))
    
    def run(self):
        ok = False
        self._printBanner()
        print("")
        question = "Do you want to install a new station on this PC (answer 'no' if you just updated %s)"%misc.getAppName()
        yes = misc.queryYesNo(question, default="no")
        if not yes:
            return

        appFolder = misc.getDefaultAppFolder()
        question = "Do you want to use the default workspace %s for this station"%(appFolder)
        ret = misc.queryYesNo(question)
        if ret == False:
            appFolder = misc.queryFolder("Please specify the installation folder: ")
            print ("... installing into %s"%appFolder)
            
        def stationExists(value):
            value = str(value)
            if Station.Station.stationExists(appFolder, value):
                question = "Station already exists, do you want to override the station (WARNING: all settings of this station will be lost)"
                yes = misc.queryYesNo(question, default="no")
                return yes
            else:
                try:
                    os.makedirs(os.path.join(appFolder, value))
                    os.makedirs(os.path.join(appFolder, value, "repo"))
                    rv = True
                except Exception as e:
                    print ("could not create folder %s, error message: %s"%(value, str(e)))
                    rv = False
            return rv
            
        stationName = misc.queryString("Please give the station name: ", cb=stationExists)
        ok, scmConfig = self.promptSCM(appFolder, stationName)
        if ok:
            ok, connectorName, connectorConfig = self.promptConnection()
        if ok:
            print("-------------------------------------------------------------")
            print("installing station now...")
            config = {}
            config["SCM"] = scmConfig
            config["connectors"] = {}
            config["connectors"]["connectorName"] = connectorName
            config["connectors"]["connectorConfig"] = connectorConfig
            station = Station.Station.createStation(appFolder, stationName, config)
            if station != None:
                station.saveStation()
            else:
                print ("something went wrong, see log for more details")
        
    def promptSCM(self, appFolder, stationName):
        ok = True
        auth = {}
        print("")
        print("-------------------------------")
        print("Configuration of git connection")
        print("")
        print("Set username and email address")
        username = misc.queryString("Please specify your git username: ")
        email = misc.queryString("Please specify your git email: ")
        print("")
        print("Selection Authentication method")
        print("-------------------------------------------------------------")
        print("*         SSH key")
        print("          username/password (not implemented yet)")
        print("-------------------------------------------------------------")
        print("")
        question = ("Press enter to keep the current choice[*] "
                    "or type selection number: ")
        rv = misc.queryChoice(question, choice=[0], default=0, typ=int)
        if rv == 0:
            auth["AuthType"] = "ssh"
            auth["Type"] = "git"
            auth["username"] = username
            auth["email"] = email
            repoFolder = os.path.join(appFolder, stationName, "repo")
            url = misc.queryString("Please specify the remote URL to use: ",
                                   cb=partial(_checkGitRemote, repoFolder, stationName))
            auth["remote"] = url
        return ok, auth
        
    def promptConnection(self):
        m = ConnectionManager.ConnectionManager()
        print("")
        print("-------------------------------------------------------------")
        print("The following connectors are available to create this station")
        print("")
        print("Selection Description")
        print("-------------------------------------------------------------")
        idxs = []
        for idx, name in enumerate(m.getConnectors()):
            
            if idx == 0:
                printStr = "*% 8d %s"%(idx, name)
            else:
                printStr = "% 9d %s"%(idx, name)
                
            print(printStr)
            idxs.append(idx)
     
        print("")
        question = "Press enter to keep the current choice[*] or type selection number: "
        rv = misc.queryChoice(question, choice=idxs, default=idxs[0], typ=int)
        connectorName = m.getConnectors()[rv]
        ok = True
        connectorConfig = {}
        cfg = m.getConnectorConfigDesc(connectorName)
        connectorConfig = self.prompConnectorCfg(cfg)
        return ok, connectorName, connectorConfig
        
    def prompConnectorCfg(self, config):
        connectorCfg = {}
        if len(config) > 0:
            print("-------------------------------------------------------------")
            print("This connector requires additional configuration")
            print("-------------------------------------------------------------")
        for cfg in config:
            question = "%s (%s): "%(cfg.name, cfg.description)
            if isinstance(cfg, Types.Path):
                val = misc.queryFolder(question)
            elif isinstance(cfg, Types.String):
                val = misc.queryString(question)
            elif isinstance(cfg, Types.Password):
                val = misc.queryPassword(question)
            elif isinstance(cfg, Types.Int):
                val = misc.queryInteger(question, minVal=cfg.minVal, maxVal=cfg.maxVal)
            else:
                raise RuntimeError("unsupported type " + str(cfg))
            connectorCfg[cfg.name] = val
        return connectorCfg


from enum import Enum
import appdirs
import pkg_resources
import sys
import os
import getpass

class License(Enum):
    Type = 1

class Links(Enum):
    Documentation = 1
    SourceCode = 2
    
def getAppName():
    return pkg_resources.require("openProduction")[0].project_name
    
def getAuthorName():
    return "pieye"
    
def getDefaultAppFolder():
    appname = getAppName()
    appauthor = getAuthorName()
    return appdirs.user_data_dir(appname, appauthor)
    
def getVersion():
    version = pkg_resources.require("openProduction")[0].version
    return version

def getLink(link):
    rv = ""
    if link == Links.SourceCode:
        rv = "https://github.com/Coimbra1984/openProduction"
    elif link == Links.Documentation:
        rv = "https://readthedocs.org/projects/openProduction/"
    return rv

def getLicense(lic):
    rv = ""
    if lic == License.Type:
        rv = "GPLv3"
    return rv

def queryYesNo(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")
         
def queryFile(question):
    while True:
        sys.stdout.write(question)
        file = input().lower()
        file = os.path.abspath(file)
        if os.path.isfile(file):
            break
        else:
            print("File %s doesn't exist, please give an existing file"%file)
            continue
                
            break
    return file            
            
def queryFolder(question):
    while True:
        sys.stdout.write(question)
        path = input().lower()
        path = os.path.abspath(path)
        if os.path.exists(path):
            break
        else:
            try:
                os.makedirs(path)
            except Exception as e:
                print ("could not create folder %s, error message: %s"%(path, str(e)))
                continue
                
            break
    return path

def queryString(question, cb=None):
    while True:
        sys.stdout.write(question)
        value = input().lower()
        if cb != None:
            if cb(value):
                break
            else:
                continue
        break
    return value

def queryInteger(question, minVal=None, maxVal=None):
    if minVal != None and maxVal != None:
        if minVal > maxVal:
            raise ValueError("maxVal must be equal-or-greater than minVal")
        prompt = "[%d..%d]: "%(minVal,maxVal)
    else:
        minVal = None
        maxVal = None
        prompt = ""
        
    while True:
        sys.stdout.write(question+prompt)
        value = input().lower()
        try:
            value=int(value)
            ok = True
        except:
            ok = False
        
        if ok:
            if minVal == None:
                break
            else:
                if value>=minVal and value<=maxVal:
                    break
        
        if minVal != None:
            inRange = "in range [%d..%d]"%(minVal, maxVal)        
        else:
            inRange = ""
        sys.stdout.write("Please give a number %s.\n"%inRange)
        
    return value


def queryPassword(question, cb=None):
    while True:
        pw1 = getpass.getpass(question)
        pw2 = getpass.getpass("Type in the same pasword again: ")
        if pw1 == pw2:
            break
        else:
            print("Passwords did not match, please try again")
    return pw1

def queryChoice(question, choice, default=None, typ=str):
    if default not in choice:
        raise ValueError("invalid default answer: '%s'" % str(default))
    for c in choice:
        if type(c) != typ:
            raise ValueError("choice contains invalid types")
        
    while True:
        sys.stdout.write(question)
        inp = input().lower()
        if default is not None and inp == '':
            rv = default
            break
        else:
            try:
                rv = typ(inp)
                if rv in choice:
                    break
            except:
                pass
            
            sys.stdout.write("Invalid choice, use one of %s\n"%str(choice))
    
    return rv


from openProduction.suite import TestSuite
from openProduction.product import ProductExecutor
from openProduction.station import Station
from openProduction.common import misc
from openProduction.connectors.BaseConnector import ConnectorErrors
import logging 

class ResultConnector(ProductExecutor.ProductExecutor):
    """Result connector setup hooks"""
    
    def __init__(self, ctor):
        super(ResultConnector, self).__init__(ctor)
        self.logger = logging.getLogger(misc.getAppName())
        self.station = Station.Station.getInstance()
        
        if self.station != None:
            self.stationName = self.station.stationName
            
    def installRunner(self, runner):
        self.testSuiteRunner = runner
        
    def install(self, testSuite):
        hint = TestSuite.InstallHints(shouldBeLast=True)
        testSuite.addMethodHint(self.teardown_result_00100, hint)
        self.testSuite = testSuite

    def teardown_result_00100(self, params, values):
        """Durchlaufergebnisse ablegen"""
        if self.station != None and self.station.loadedProduct != None:
            if "NO_TRANSMIT_RESULTS" in params:
                self.logger.info("Skippe Erbenisübertragung, 'NO_TRANSMIT_RESULTS' in params gesetzt, Parameter sind:")
                self.logger.info(str(values))
                return TestSuite.ExecutionResult.SKIP
            else:
                if "NO_LOG_RESULTS" not in params:
                    f = open("log.txt", "w")
                    f.write(self.testSuiteRunner.suiteResults.getLogText())
                    f.close()
                    values.addFile("log.txt")
                
                values.move("result")
                values["revision_id"] = self.station.loadedProduct.version.revision
                values["station_id"] = self.station.stationID
                values["duration"] = self.testSuiteRunner.suiteResults.executionTime
                values["testCaseResults"] = self.testSuiteRunner.suiteResults.toDict()
                values["openProductionVersion"] = misc.getVersion()
                values["success"] = self.testSuiteRunner.suiteResults.isSuccessful()
                values["deviceID"] = values.getDeviceID()
                values["tempParams"] = self.tempParams.getParams()
                self.logger.info("Übertrage Ergebnisse")
                rv, data = self.station.serverIF.createProductResult(values.getParams())
                self.assertEqual(rv, ConnectorErrors.NO_ERROR, "Fehler beim Speichern der Durchlaufergebnisse")
                result_id = data["product_result_id"]
                
                for f in values.getFiles():
                    if "sftp_config_id" in params:
                        sftp_config_id = params["sftp_config_id"]
                    else:
                        sftp_config_id = 1
                    data = {"result_id":result_id, "filename": f, "sftp_config_id": sftp_config_id}
                    rv = self.station.serverIF.createProductResultFile(data)
                    self.logger.info("Übertrage Datei %s"%f)
                    self.assertEqual(rv, ConnectorErrors.NO_ERROR, "Fehler beim Speichern der Datei %s"%f)
        else:
            if self.station == None:
                self.logger.info("Skippe Erbenisübertragung, station=None")
            else:
                self.logger.info("Skippe Erbenisübertragung, station.loadedProduct=None")
            return TestSuite.ExecutionResult.SKIP
        


import openProduction.connectors.BaseConnector as plugintypes
import os
import shutil
import pandas

class FileSystemConnector(plugintypes.BaseConnector):    
    def __init__(self):
        __fields__ = []
        super(FileSystemConnector, self).__init__(__fields__)
    
    def createDatabase(self, dbname):
        path = os.path.join(self.workDir, dbname)
        if not os.path.exists(path):
            os.makedirs(path)
            pandas.DataFrame().to_hdf(os.path.join(path+"tables.hdf5"), "_init_")
    
    def deleteDatabase(self, dbname):
        path = os.path.join(self.workDir, dbname)
        shutil.rmtree(path)
        return True
        
    def createTable(self, dbName, tableName, errorIfExists=True):
        file = os.path.join(self.workDir, dbName, "tables.hdf5")
        if os.path.exists(file):
            store = pandas.HDFStore(file,'r')
            if ("/"+tableName in store.keys()) and errorIfExists==True:
                store.close()
                raise RuntimeError("Table %s already exists"%tableName)
            elif ("/"+tableName not in store.keys()):
                store.close()
                pandas.DataFrame().to_hdf(file, tableName)
        elif os.path.exists(file) == False:
            self.createDatabase(dbName)
            pandas.DataFrame().to_hdf(file, tableName)
        return True
        
    def listDatabases(self):
        dbs = []
        for f in os.listdir(self.workDir):
            absF = os.path.join(self.workDir, f)
            if os.path.isdir(absF):
                dbs.append(os.path.split(absF)[-1])
        return dbs

    def addRow(self, dbName, table, entries):
        fname = os.path.join(self.workDir, dbName, table + ".hdf5")
        rows = pandas.read_hdf(fname)
        
        return ok
        raise NotImplementedError("addRow method has to be defined in connector's plugin")
    
    def getColumns(self, dbName, table, columnName):
        raise NotImplementedError("getColumns method has to be defined in connector's plugin")    
    
    def getDescription(self):
        return "This connector uses your local file system"
    
    def _onNewConfig(self):
        ok = True
        
        if not os.path.exists(self.workDir):
            os.makedirs(self.workDir)
        return ok


import threading
import logging
from openProduction.common import misc, Signals, Version
from enum import Enum
from openProduction.suite import TestSuite, TestLoader, TestClass, TestSuiteIO
from openProduction.suite.TestSuite import SuiteState
from openProduction.product import ProductExecutorCtor
from concurrent.futures import ThreadPoolExecutor
import os
from openProduction.common import Exceptions
from openProduction.connectors.BaseConnector import ConnectorErrors

class HWClass:
    
    def __new__(cls, cla, errMsg, *args, **kwargs):
        cls._inst = None
        instance = super(HWClass, cls).__new__(cls)
        return instance
        
    def __init__(self, cla, errMsg, *args, **kwargs):
        self._cla = cla
        self._args = args
        self._kwargs = kwargs
        self.errMsg = errMsg
        
    def getErrMsg(self):
        return self.errMsg
        
    def construct(self):
        if "initCallback" in self._kwargs:
            cb = self._kwargs["initCallback"]
            self._kwargs.pop("initCallback", None)
        else:
            cb = None
        self._inst = self._cla(*self._args, **self._kwargs)
        
        if cb != None:
            cb(self._inst)
        
        return self._inst
    
    def __setattr__(self, name, value):
        if self._inst != None:
            setattr(self._inst, name, value)
        else:
            self.__dict__[name] = value
    
    def __getattr__(self, name):
        return getattr(self._inst, name)


class HardwareManager:
    
    instance = None
    
    def __init__(self):
        if not HardwareManager.instance:
            HardwareManager.instance = HardwareManager.__HardwareManager()
            
    def __getattr__(self, name):
        return getattr(self.instance, name)
    def __setattr__(self, name, value):
        return setattr(self.instance, name, value)
    
    class __HardwareManager:
        def __init__(self):
            self.hwList = []
            
        def add(self, cla, errMsg, *args, **kwargs):
            hw = HWClass(cla, errMsg, *args, **kwargs)
            self.hwList.append(hw)
            return hw
            
        def probe(self):
            ok = True
            desc = ""
            
            for hw in self.hwList:
                
                try:
                    obj = hw.construct()
                except:
                    ok = False
                    desc = hw.getErrMsg()
                    break
                
                if hasattr(obj, "probe"):
                    ok, desc = hw.probe()
                    if ok == False:
                        break
    
            return ok, desc
    
        def stop(self):
            for hw in self.hwList:
                if hasattr(hw, "close"):
                    hw.close()
            self.hwList = []
            
    @staticmethod
    def getInstance():
        return HardwareManager.instance

class ProductTrigger(threading.Thread):

    def __init__(self, runner, executor):
        threading.Thread.__init__(self)
        self.executor = executor
        self.hardwareManager = self.executor.hardwareManager
        self.runner = runner
        self.wasStarted = False
        self.runner.sequenceComplete.connect(self.onSequenceComplete)
        self._stop_event = self.executor._unload_event
    
    def onSequenceComplete(self, stepType, result):
        if stepType == SuiteState.LOADING and result.isSuccessful():
            self.startTrigger()
            self.wasStarted = True
        if stepType == SuiteState.UNLOADING and self.wasStarted == True:
            self.stopTrigger()
            
    def startTrigger(self):
        self.start()
        
    def stopTrigger(self):
        self.join()
        
class ProductExecutor(TestClass.TestClass):
    
    def __init__(self, ctor):
        super(ProductExecutor, self).__init__()
        if isinstance(ctor, ProductExecutorCtor.ProductExecutorCtor) == False:
            raise RuntimeError("ctor must be of type openProduction.product.ProductExecutor.ProductExecutorCtor")
        self.logger = logging.getLogger(misc.getAppName())
        self.params = ctor.params
        self.values = ctor.values
        self.tempParams = ctor.tempParams
        self.ioHandler = ctor.io
        self.ui = ctor.ui
        if self.ioHandler == None:
            self.ioHandler = TestSuiteIO.BaseIOHandler()
        self.hardwareManager = HardwareManager()

        #check if product has a previous station
        if self.values != None:
            from openProduction.station import Station
            station = Station.Station.getInstance()
            if station != None:
                product = station.loadedProduct
                if product != None:
                    self.values.deviceIDSet.connect(self._onUniqueCheck)
                    stationID = station.stationID
                    productID = product.productID
                    serverIF = station.serverIF
                    rv, prevID = serverIF.getPreviousStationID(stationID, productID)
                    if rv == ConnectorErrors.NO_ERROR and prevID != None:
                        self.logger.info("found previous station_id=%d"%prevID)
                        self.previousStationID = prevID
                        rv, data = serverIF.getStation(prevID)
                        if rv == ConnectorErrors.NO_ERROR:
                            self.previousStationName = data["name"]
                            self.values.deviceIDSet.connect(self._onPreviousStationCheck)
                        else:
                            raise RuntimeError ("query station name failed with %s"%rv.name)
                else:
                    self.logger.info("product is None, skipping previous station check")
            else:
                self.logger.info("station is None, skipping previous station check")                            
                            
        self.tempParams["Überprüfe Vorgängerstation"] = True
        self.tempParams["Einmaliger Durchlauf"] = False
        
    def abort(self):
        self._abort_event.set()
        self.ioHandler.abort()
        
    def abortClear(self):
        self._abort_event.clear()
        self.ioHandler.abortClear()
        
    def _onPreviousStationCheck(self, deviceID):        
        if self.tempParams["Überprüfe Vorgängerstation"] == False:
            return
        
        from openProduction.station import Station
        serverIF = Station.Station.getInstance().serverIF
        filt = "`station_id`=%d and `success`=1 and `deviceID` like '%s'"%(self.previousStationID, deviceID)
        rv, data = serverIF.listProductResults(filt)
        self.assertEqual(rv, ConnectorErrors.NO_ERROR, "Gerät %s nicht durch Station %s gelaufen"%(deviceID, self.previousStationName))
        
    def probe(self):
        return self.hardwareManager.probe()
    
    def logAndAssertInRange(self, value, key, msg=None):
        self.values[key] = value
        valMin = self.params["min"+key]
        valMax = self.params["max"+key]
        self.assertInRange(value, valMin, valMax, msg=msg)
        
    def _onUniqueCheck(self, deviceID):
        self.assertUnique()
            
    def assertUnique(self):
        if self.tempParams["Einmaliger Durchlauf"] == False:
            return
        from openProduction.station import Station
        deviceID = self.values.getDeviceID()
        station = Station.Station.getInstance()
        self.assertNotEqual(station, None, "keine Station geladen")
        stationID =station.stationID
        filt = "`station_id`=%d and `success`=1 and `deviceID` like '%s'"%(stationID, deviceID)
        rv, data = station.serverIF.listProductResults(filt)
        self.assertEqual(rv, ConnectorErrors.NOT_FOUND, "Gerät %s schon durch Station gelaufen"%deviceID)

class ProductRunner:
    
    class RunnerState(Enum):
        INIT = 0
        SETUP = 1
        RUN = 2
        TEARDOWN = 3
    
    def __init__(self, productDir, productName, params, ioHandler=None, ui=None):
        
        self._pool = ThreadPoolExecutor(3)
        
        self.logger = logging.getLogger(misc.getAppName())
        self.triggered = Signals.Signal()
        
        #at this stage, don't construct anything
        self.params = params
        self._ioHandler = ioHandler
        self._ui = ui
        
        self.suite = TestSuite.TestSuite(productName, "", params=self.params, values=None, tempParams=None)
        self.suiteRunner = TestSuite.TestSuiteRunner(self.suite, stopOnFail=True, stopOnExcept=True)
        
        self.suiteRunner.sequenceComplete.connect(self._onSequenceComplete)
        
        
        params.setReadOnly(False)
        if "productHookDir" not in params:
            params["productHookDir"] = productDir
        else:
            params["productHookDir"] = os.path.abspath(os.path.join(productDir, params["productHookDir"]))
            
        if "productHookPattern" not in params:
            params["productHookPattern"] = ["hook_*.py"]
        if "productHookRegularPattern" not in params:
            params["productHookRegularPattern"] = "step_"
        if "productHookSetupPattern" not in params:
            params["productHookSetupPattern"] = "setup_"
        if "productHookTearDownPattern" not in params:
            params["productHookTearDownPattern"] = "teardown_"
        if "productHookLoadPattern" not in params:
            params["productHookLoadPattern"] = "load_"
        if "productHookUnloadPattern" not in params:
            params["productHookUnloadPattern"] = "unload_"
        if "skipOpenProductionHooks" in params:
            myFolder = productDir
        else:
            myFolder = os.path.split(os.path.abspath(__file__))[0]
        params.setReadOnly(True)
            
        loader = TestLoader.TestDirectoryLoader(myFolder)
        ctor = ProductExecutorCtor.ProductExecutorCtor(self._ioHandler, self.params, self.suite.values, self.suite.tempParams, self._ui)
        tgs = loader.discover(ctor)
        for tg in tgs:
            self.suite.addTestCaseGroup(tg)
        
        self._state = ProductRunner.RunnerState.INIT
                
    def _onNewTrigger(self):
        if self._state == ProductRunner.RunnerState.RUN:
            self.triggered.emit()
            rv = self.suiteRunner.run()
        else:
            self.logger.warning("can't trigger ProductRunner in state %s"%self._state)
            rv= None
            
        return rv
    
    def trigger(self):
        return self._onNewTrigger()
    
    
    def _unloadSuiteAfterErr(self):
        self._loadFuture.result(timeout=10)
        self.unload()
                    
    def _onSequenceComplete(self, executionType, suiteResults):
        if executionType == TestSuite.SuiteState.LOADING:
            self._state = ProductRunner.RunnerState.RUN
            if suiteResults.isSuccessful() == False:                
                self._pool.submit(self._unloadSuiteAfterErr)
        if executionType == TestSuite.SuiteState.UNLOADING:
            self._state = ProductRunner.RunnerState.INIT
    
    def load(self, startTrigger=True, startMonitor=True):
        if self._state == ProductRunner.RunnerState.INIT:
            self._state = ProductRunner.RunnerState.SETUP
            self._loadFuture = self.suiteRunner.loadSuite()
            return self._loadFuture
        else:
            self.logger.warning("can't start ProductRunner in state %s"%self._state)
            return None
        
    def unload(self):
        if self._state == ProductRunner.RunnerState.RUN:
            return self.suiteRunner.unloadSuite()
        else:
            self.logger.warning("can't stop ProductRunner in state %s"%self._state)
            return None
    


import logging
from openProduction.common import misc
from openProduction.station import StationLoader
import glob
import os

class StationManager:

    def __init__(self, workspace):
        self.logger = logging.getLogger(misc.getAppName())
        self.workspace = workspace

    def listStations(self):
        self.logger.info("looking for stations in path %s"%self.workspace)
        globExpr = os.path.join(self.workspace, "*", "config.json")
        stations = []
        for filename in glob.iglob(globExpr):
            stationName = os.path.split(os.path.relpath(filename, self.workspace))[0]
            
            self.logger.info("found candidate '%s'"%stationName)
            s = StationLoader.StationLoader(self.workspace, stationName)
            if s.load():
                stations.append(stationName)
            
        return stations


class ProductExecutorCtor:
    def __init__(self, io, params, values, tempParams, ui):
        self.io = io
        self.params = params
        self.values = values
        self.tempParams = tempParams
        self.ui = ui


from setuptools import setup, find_packages
from setuptools.command.install import install
import logging

setup(name='openProduction',
      version='0.0.17',
      description='A framework to model device production',
      url='https://github.com/Coimbra1984/openProduction',
      author='Markus Proeller',
      author_email='markus.proeller@pieye.org',
      license='GPLv3',
      include_package_data=True,
      install_requires=[
        "appdirs", "yapsy", "PyMySQL", "cmd2", "GitPython", "PyQtChart", 
        "pysmb", "pandas", "colorama"
      ],
      packages=find_packages(),
      test_suite="openProduction.tests",
      package_data={'': ['*.py', '*.yapsy-plugin'] },
      zip_safe=False)


# define Python user-defined exceptions
class OpenProductionAssertionError(Exception):
   """Base class for other exceptions"""
   def __init__(self, message):
        # Call the base class constructor with the parameters it needs
        super(OpenProductionAssertionError, self).__init__(message)


class OpenProductionAbortError(Exception):
   """Base class for other exceptions"""
   def __init__(self, message):
        # Call the base class constructor with the parameters it needs
        super(OpenProductionAbortError, self).__init__(message)



from openProduction.product import ProductExecutor, ProductExecutorCtor
from openProduction.suite import TestLoader, TestParams

class ProductSetUp(ProductExecutor.ProductExecutor):
    """Product setup hooks"""
    
    def __init__(self, ctor):
        super(ProductSetUp, self).__init__(ctor)
        
    def load__00050(self, params, values):
        """Executing self tests"""
        import unittest
        import openProduction.tests.suite_test as myTests
        suite = unittest.TestLoader().loadTestsFromModule(myTests)
        results = unittest.TestResult()
#        suite.run(results)
#        results = unittest.TextTestRunner(verbosity=2).run(suite)
        
#        self.assertEqual(results.wasSuccessful(), True)
        
    def install(self, testSuite):
        self.testSuite = testSuite

    def load__00100(self, params, values):
        """Construct product executors"""
        ctor = ProductExecutorCtor.ProductExecutorCtor(self.ioHandler, params, values, self.tempParams, self.ui)
        folder = params["productHookDir"]
        l = TestLoader.TestDirectoryLoader(folder)
        tcg = l.discover(ctor, 
                   pattern=params["productHookPattern"],
                   testClass=ProductExecutor.ProductExecutor,
                   patternRegular=params["productHookRegularPattern"],
                   patternSetup=params["productHookSetupPattern"],
                   patternTearDown=params["productHookTearDownPattern"],
                   patternLoad=params["productHookLoadPattern"],
                   patternUnload=params["productHookUnloadPattern"])
        
        return tcg
        
    def load__00200(self, params, values):
        """Probing product executors"""
        ok, desc = self.hardwareManager.probe()
        self.assertEqual(ok, True, "Probing HW failed, with msg %s"%(desc))
        


import logging
from openProduction.common import misc
from git import Repo
import collections
import os
import shutil
import urllib.parse

def createHierarchy(d, u):
    for k, v in u.items():
        if isinstance(v, collections.Mapping):
            d[k] = createHierarchy(d.get(k, {}), v)
        else:
            if k not in d:
                d[k] = v
    return d

class SCM:
    
    def __init__(self, scmDir, config):
        self.config = config
        self.logger = logging.getLogger(misc.getAppName())
        self.username = config["git_username"]
        self.email = config["git_usermail"]
        
        project_dir = os.path.dirname(os.path.abspath(__file__))
        os.environ['GIT_ASKPASS'] = os.path.join(project_dir, 'askpass.py')
        os.environ['GIT_USERNAME'] = config["git_username"]
        os.environ['GIT_PASSWORD'] = config["git_password"]
        
        if os.path.exists(scmDir) == False:
            self.logger.info("repo directory %s doesn't exist, cloning again"%scmDir)
            os.makedirs(scmDir)
            url = self.createURL(config)
            Repo.clone_from(url, scmDir)
        
        self.scmDir = scmDir
        self.repo = Repo.init(scmDir)
        self.repo.config_writer().set_value("user", "name", self.username).release()
        self.repo.config_writer().set_value("user", "email", self.email).release()
        
        if len(self.repo.remotes) == 0:
            self.logger.info("repo has no remote information, adding url %s"%config["git_url"])
            url = self.createURL(config)
            self.repo.create_remote('origin', url)
                        
        if self.getURL() != self.createURL(self.config):
            self.logger.info("remote url in local working copy doesn't match. Wipe out working copy %s"%self.scmDir)
            self.repo.close()
            shutil.rmtree(self.scmDir)
            os.makedirs(self.scmDir)
            Repo.clone_from(self.createURL(self.config), self.scmDir)
            self.repo = Repo.init(self.scmDir)            
        
        self.logger.info("fetching changes from remote")
        self.repo.remotes.origin.fetch()
        
        self.createGitignore(self.scmDir)
        self.repo.git.add(".gitignore")
        
    def createURL(self, config):
        proto, url = config["git_url"].split("//")
        username = urllib.parse.quote(config["git_username"])
        password = urllib.parse.quote(config["git_password"])
        url = proto + "//" + username + ":" + password + "@" + url
        return url
        
    def getURL(self):
        return self.repo.remotes.origin.url
        
    def createGitignore(self, folder):
        ignore = ["__pycache__/", "*.py[cod]", "*$py.class"]
        
        fname = os.path.join(folder, ".gitignore")
        if os.path.exists(fname) == False:
            f = open(fname, "w")
            f.write('\n'.join(ignore) + '\n')
            f.close()
        else:
            wlines = []
            #check if exists
            f = open(fname, "r+")
            lines = f.read().splitlines()
            for i in ignore:
                if i not in lines:
                    wlines.append(i)
            for w in wlines:
                f.write(w+"\n")
            f.close()
        
    def isDirty(self):
        return self.repo.is_dirty()
    
    def scmDirHasChanges(self):
        diff = self.repo.git.diff(self.scmDir)
        if diff == '':
            changed = False
        else:
            changed = True
        return changed
            
    def checkoutProductRevision(self):
        if self.getURL() != self.createURL(self.config):
            self.logger.info("remote url in local working copy doesn't match. Wipe out working copy")
            shutil.rmtree(self.scmDir)
            os.makedirs(self.scmDir)
            Repo.clone_from(self.createURL(self.config), self.scmDir)
            self.repo = Repo.init(self.scmDir)
            
        self.repo.git.checkout(self.config["commit_id"])
        
    def getBaseDir(self):
        return self.scmDir
    
    def addFile(self, file):
        self.repo.index.add([file])



import os
import argparse
import sys
import log
from openProduction.common import misc

logger = log.initLogger()


my_path = os.path.split(os.path.abspath(__file__))[0]
#sys.path.append(os.path.join(my_path, "common"))

parser = argparse.ArgumentParser(description='openProduction')
parser.add_argument('--verbose', '-v', action='store_true')
parser.add_argument('--headless', action='store_true', help='run without GUI')
parser.add_argument('--workspace', type=str, default=misc.getDefaultAppFolder(), help='workspace directory, defauts to %s'%misc.getDefaultAppFolder())
parser.add_argument('--stationCLI', action='store_true', default=False)

args = parser.parse_args()

if args.headless == False:
    from PyQt5 import QtWidgets, QtGui
    from openProduction.UI import mainUI
    sys.argv += ['--style', 'material']
    app = QtWidgets.QApplication(sys.argv)
    app.setWindowIcon(QtGui.QIcon(":/icons/openProductionNurLogo-transparent.png"))
    ui = mainUI.UIRunner(args.workspace)
    app.exec_()
    
    

sys.exit(0)